/*
 This example has been adapted from the one taken from:
 T Muldner, C++ Programming with Design Patterns Revealed, Addison-Wesley,2002
 */

#include <iostream>

using namespace std;

class Singleton {
public:
    static Singleton* instance();
    void updateSingletonData(int);
    void printSingletonData();
protected:
    Singleton();
    Singleton(const Singleton&);
    Singleton& operator=(const Singleton&);
    //~Singleton();
    virtual ~Singleton();
private:
    static Singleton* onlyInstance_;
    int               singletonData;
};

void Singleton::updateSingletonData(int i){
    singletonData = i;
}

void Singleton::printSingletonData(){
    cout<<"Singleton data: "<<singletonData<<endl;
}

Singleton::Singleton(){
    singletonData = 0;
}

Singleton::Singleton(const Singleton& s) {}
Singleton& Singleton::operator=(const Singleton& s) {}
Singleton::~Singleton(){cout<<"Destructor"<<endl;}

Singleton* Singleton::onlyInstance_ = 0;

Singleton* Singleton::instance() {
    if(onlyInstance_ == 0) {
        onlyInstance_ = new Singleton();
    }
    return onlyInstance_;
} 












int main(){
    /*
     Singleton* s = Singleton::instance();
     s->printSingletonData();
     s->updateSingletonData(10);
     s->printSingletonData();
     Singleton* t = Singleton::instance();
     t->printSingletonData();
     return 0;
     */
    
    // More correct usage of Singleton would be:
    Singleton::instance()->printSingletonData();
    Singleton::instance()->updateSingletonData(100);
    Singleton::instance()->printSingletonData();
}








































/*
 This example has been adapted from the one taken from:
 T Muldner, C++ Programming with Design Patterns Revealed, Addison-Wesley,2002
 
 Adapted Muldner Singleton, follows the Meyers Singleton principle to remove the 
 global inisialisation of the onlyInstance variable.  This version makes use of 
 references rather than pointers
 
 */

#include <iostream>

using namespace std;

class Singleton {
public:
    static Singleton& instance();  //static Singleton* instance();
    void updateSingletonData(int);
    void printSingletonData();
protected:
    Singleton();
    Singleton(const Singleton&);
    Singleton& operator=(const Singleton&);
    ~Singleton();
private:
    // No longer required: static Singleton* onlyInstance_;
    int               singletonData;
};

void Singleton::updateSingletonData(int i){
    singletonData = i;
}

void Singleton::printSingletonData(){
    cout<<"Singleton data: "<<singletonData<<endl;
}

Singleton::Singleton(){
    singletonData = 0;
}

Singleton::Singleton(const Singleton& s) {}
Singleton& Singleton::operator=(const Singleton& s) {}
Singleton::~Singleton(){}

// No longer required: Singleton* Singleton::onlyInstance_ = 0;

/* Instance function changes
 Singleton* Singleton::instance() {
 if(onlyInstance_ == 0) {
 onlyInstance_ = new Singleton();
 }
 return onlyInstance_;
 } 
 */

Singleton& Singleton::instance(){
    static Singleton onlyInstance_;
    return onlyInstance_;
}

int main(){
    /*
     Singleton& s = Singleton::instance(); //Singleton* s = Singleton::instance();
     s.printSingletonData();              //s->printSingletonData();
     s.updateSingletonData(10);           //s->updateSingletonData(10);
     s.printSingletonData();              //s->printSingletonData();
     Singleton& t = Singleton::instance(); //Singleton* t = Singleton::instance();
     t.printSingletonData();              //t->printSingletonData();
     return 0;
     */
    
    // More correct usage of Singleton would be:
    Singleton::instance().printSingletonData();
    Singleton::instance().updateSingletonData(100);
    Singleton::instance().printSingletonData();
}


